/*
 * Copyright 2013 Graham Smith
 */
package uk.co.wobblycogs.samplefx4;

import java.io.IOException;
import java.net.URL;
import java.util.ResourceBundle;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.event.Event;
import javafx.event.EventHandler;
import javafx.event.EventType;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.Node;
import javafx.scene.control.ContextMenu;
import javafx.scene.control.ListView;
import javafx.scene.control.MenuItem;
import javafx.scene.control.TreeCell;
import javafx.scene.control.TreeItem;
import javafx.scene.control.TreeView;
import javafx.scene.control.cell.TextFieldTreeCell;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.ClipboardContent;
import javafx.scene.input.DataFormat;
import javafx.scene.input.DragEvent;
import javafx.scene.input.Dragboard;
import javafx.scene.input.MouseEvent;
import javafx.scene.input.TransferMode;
import javafx.util.Callback;
import javafx.util.StringConverter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This class is set as the controller for the FXML resource that handles
 * organising people into departments.
 * <p>
 * If this is the only place where {@code Department} and {@code Person} objects
 * are used in the application (which is the case here) this makes a reasonably
 * good argument for this class to also act as the data model.
 * <p>
 * If the {@code Department} and {@code Person} objects are used widely
 * throughout the application though a separate model consisting of one or more
 * classes would be a better design to use.
 *
 * @author Graham Smith
 */
public class CompanyManagerController implements Initializable {

	public static final DataFormat PERSON_DATA_FORMAT = new DataFormat("uk.co.wobblycogs.samplefx4.Person");
	private Logger logger = LoggerFactory.getLogger(getClass());
	private DepartmentService departmentService;
	private PersonService personService;
	@FXML
	private TreeView<EntityProxy> departmentTree;
	@FXML
	private ListView<Person> personList;
	private ObservableList<Person> unassigned;
	private ContextMenu departmentMenu = new ContextMenu();

	public CompanyManagerController() {
		try {
			departmentService = new DepartmentService(Demo.PERSISTENCE_UNIT_NAME);
			personService = new PersonService(Demo.PERSISTENCE_UNIT_NAME);

			Department wholeCompany = departmentService.findRoot();
			if (wholeCompany == null) {
				logger.info("No current root department found so creating one.");
				wholeCompany = new Department();
				wholeCompany.setName("Whole Company");
				departmentService.persist(wholeCompany);
			}

			logger.info("Got a root department of: {}", wholeCompany.getId());
		} catch (IOException ex) {
			throw new RuntimeException("Failed to create controller", ex);
		}

		createDepartmentContextMenu();
	}

	private void createDepartmentContextMenu() {
		//Context menu for manipulating departments.
		MenuItem addDepartmentItem = new MenuItem("Add Department");
		departmentMenu.getItems().add(addDepartmentItem);
		addDepartmentItem.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(ActionEvent t) {
				//Firstly create the new department. In a real application you
				//would probably want to pop up a dialog at this point to
				//capture some information such as the department name.
				Department child = new Department();
				child.setName("New Department");
				departmentService.persist(child);

				//Then find the selected node and add the new child
				TreeItem<EntityProxy> selected = departmentTree.getSelectionModel().getSelectedItem();
				Department parent = departmentService.find(selected.getValue().getId());
				departmentService.addChildDepartment(parent, child);

				//Finally place the node in the tree.
				TreeItem<EntityProxy> newDepartment = new TreeItem<>(new EntityProxy(child.getId(), EntityProxyType.DEPARTMENT, child.getName()), new ImageView(EnhancedTextFieldTreeCell.DEPARTMENT_ICON));
				selected.getChildren().add(newDepartment);
			}
		});

		MenuItem deleteDepartmentItem = new MenuItem("Delete Department");
		departmentMenu.getItems().add(deleteDepartmentItem);
		deleteDepartmentItem.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(ActionEvent t) {
				TreeItem<EntityProxy> selectedItem = departmentTree.getSelectionModel().getSelectedItem();
				if (selectedItem.getParent() == null) {
					return; //Can't delete the root node.
				}

				Department selectedDepartment = departmentService.find(selectedItem.getValue().getId());
				Department parentDepartment = selectedDepartment.getParent();

				//Remove the selected department from it's parent in both the
				//tree and the persisted version
				TreeItem<EntityProxy> parentItem = selectedItem.getParent();
				parentItem.getChildren().remove(selectedItem);
				parentDepartment.getChildren().remove(selectedDepartment);
				departmentService.merge(parentDepartment);

				//Finally remove the selected department and any of it's children.
				//Caution: this uses a cascade delete on child departments
				//and personnel. In the real world you probably wouldn't want
				//this. You might want to cascade the deletion of departments
				//but staff would probably be re-asigned not sacked so it would
				//probably be better to either block the delete if all the people
				//haven't been moved out of the tree or to move them into the
				//unassigned list.
				departmentService.remove(selectedDepartment);
			}
		});
	}

	@FXML
	public void handlePersonListDragDetected(MouseEvent event) {
		logger.debug("Detected drag start on person list");
		Dragboard db = personList.startDragAndDrop(TransferMode.MOVE);
		ClipboardContent content = new ClipboardContent();
		Person selected = personList.getSelectionModel().getSelectedItem();
		logger.debug("Selected Person: {}", selected);
		content.put(PERSON_DATA_FORMAT, selected.getId());
		db.setContent(content);
		event.consume();
	}

	/**
	 * Handles a drag and drop gesture appearing over the person list. This
	 * event will probably have appeared from the department tree but it might
	 * also appear from the person list itself.
	 *
	 * @param event
	 */
	@FXML
	public void handlePersonListDragOver(DragEvent event) {
		//If the data on the dragboard represents a person the
		//person list will accept it.
		if (event.getDragboard().hasContent(CompanyManagerController.PERSON_DATA_FORMAT)) {
			event.acceptTransferModes(TransferMode.MOVE);
		}
		event.consume();
	}

	/**
	 * Handles the dropping of a person onto the person list.
	 *
	 * @param event
	 */
	@FXML
	public void handlePersonListDragDropped(DragEvent event) {
		Dragboard db = event.getDragboard();
		boolean success = false;
		if (db.hasContent(CompanyManagerController.PERSON_DATA_FORMAT)) {
			//The dragboard contains the ID of a person which is
			//looked up here.
			Person person = personService.find((Long) db.getContent(CompanyManagerController.PERSON_DATA_FORMAT));
			logger.debug("Dropping Person: {}", person);

			person.setDepartment(null);
			personService.merge(person);
			unassigned.add(person);

			success = true;
		}
		event.setDropCompleted(success);
		event.consume();
	}

	@FXML
	public void handlePersonListDragDone(DragEvent event) {
		if (event.getTransferMode() == TransferMode.MOVE) {
			Person selected = personList.getSelectionModel().getSelectedItem();
			unassigned.remove(selected);
			personList.getSelectionModel().clearSelection();
		}
		event.consume();
	}

	@Override
	public void initialize(URL url, ResourceBundle rb) {
		//Populate an observable list with all the people that aren't currently
		//assigned to a department. This list is then used to populate the
		//person list.
		unassigned = FXCollections.observableArrayList(personService.findUnassigned());
		personList.setItems(unassigned);

		Department wholeCompany = departmentService.findRoot();
		TreeItem<EntityProxy> root = growTree(wholeCompany);
		departmentTree.setRoot(root);
		root.setExpanded(true);
		//Make the tree editable so that we can update the names of departments
		//and the people that work in them.
		departmentTree.setEditable(true);
		//Put in a custom cell factory that supports editing and context menus
		departmentTree.setCellFactory(EnhancedTextFieldTreeCell.forDepartmentTreeView(departmentMenu, departmentService, personService));

		//Handle editing commits. This is where the magic happens...
		departmentTree.setOnEditCommit(
				new EventHandler<TreeView.EditEvent<EntityProxy>>() {
			@Override
			public void handle(TreeView.EditEvent<EntityProxy> item) {
				EntityProxy node = item.getNewValue();
				switch (node.getType()) {
					case DEPARTMENT:
						Department d = departmentService.find(node.getId());
						d.setName(node.getLabel());
						departmentService.merge(d);
						break;
					case PERSON:
						Person p = personService.find(node.getId());
						p.setName(node.getLabel());
						personService.merge(p);
						break;
				}
			}
		});
	}

	private TreeItem<EntityProxy> growTree(Department department) {
		TreeItem<EntityProxy> dItem = new TreeItem<>(new EntityProxy(department.getId(), EntityProxyType.DEPARTMENT, department.getName()), new ImageView(EnhancedTextFieldTreeCell.DEPARTMENT_ICON));
		dItem.setExpanded(true);
		for (Person p : department.getPersonnel()) {
			TreeItem<EntityProxy> pItem = new TreeItem<>(new EntityProxy(p.getId(), EntityProxyType.PERSON, p.getName()), new ImageView(EnhancedTextFieldTreeCell.PERSON_ICON));
			dItem.getChildren().add(pItem);
		}

		for (Department d : department.getChildren()) {
			dItem.getChildren().add(growTree(d));
		}

		return dItem;
	}
}
