/*
 * Copyright 2013 Graham Smith
 */
package uk.co.wobblycogs.samplefx4;

import java.io.IOException;
import java.text.MessageFormat;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.logging.Level;
import javafx.application.Application;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.layout.Pane;
import javafx.scene.layout.StackPane;
import javafx.stage.Stage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Main class for a demonstration application that shows one way to use JPA with
 * JavaFX. The application also includes an example of drag and drop.
 * <p>
 * Like all demonstration applications this code doesn't really do anything very
 * useful but it does, I hope, provide a template that shows how to do things
 * that are useful. The basic idea behind this application is to sort people
 * into departments and store the result in a database.
 * <p>
 * Since this application demonstrates database access it requires a database to
 * demonstrate against. The application has a persistence.xml file in the
 * META-INF directory that is configured to point at a Derby database called
 * "SampleFX4" with a username of "app" and a password of "password". To create
 * this database under NetBeans click the "Services" tab in the left upper
 * window. Expand "Databases". Right click on "Java DB" and select "Create
 * Database...". The rest should be fairly straight forward.
 * <p>
 * NetBeans has a basic database explorer that works well with the Java DB used
 * by default. If you want to watch what happens in the database as people are
 * dragged around in the front end a query like this is useful:
 * <pre>SELECT p.id, p.name, d.name FROM APP.PERSON AS p
 * LEFT JOIN APP.DEPT AS d ON p.department_id = d.id;</pre>
 *
 * @author Graham Smith
 */
public class Demo extends Application {

	private Logger logger = LoggerFactory.getLogger(getClass());
	private static Demo application;
	public static final String PERSISTENCE_UNIT_NAME = "SampleFX4PU";

	@Override
	public void init() throws Exception {
		super.init();

		//Provide a reference to the application so that controllers can share items.
		setApplication(this);

		//Start the persistance framework early. This is for two reasons, firstly
		//it lets us find problems right away and secondly it prevents a pause
		//the first time it's accessed.
		PersistenceUtilities.getInstance().getEntityManagerFactory(PERSISTENCE_UNIT_NAME, null);
	}

	@Override
	public void start(Stage stage) {
		logger.info("Starting main application");

		DemoDataGenerator ddg = new DemoDataGenerator();
		try {
			ddg.generate();
		} catch (IOException ex) {
			logger.error("Failed to generate demo data.", ex);
		}

		try {
			Pane pane = FXMLLoader.load(getClass().getResource("/resources/fxml/CompanyManager.fxml"));
			Scene scene = new Scene(pane, 1020, 720, false);
			scene.getStylesheets().add("/resources/styles/style.css");
			stage.setTitle("Company Manager - SampleFX 4");
			stage.setScene(scene);
			stage.show();
		} catch (IOException ex) {
			logger.error("Failed while creating main display.", ex);
		}
	}

	/**
	 * The main() method is ignored in correctly deployed JavaFX application.
	 * main() serves only as fallback in case the application can not be
	 * launched through deployment artifacts, e.g., in IDEs with limited FX
	 * support. NetBeans ignores main().
	 *
	 * @param args the command line arguments
	 */
	public static void main(String[] args) {
		launch(args);
	}

	/**
	 * Should be called only once at application initialisation.
	 *
	 * @param app
	 */
	private static void setApplication(Demo app) {
		application = app;
	}

	/**
	 * To be used by the rest of the application when it requires access to
	 * application features.
	 *
	 * @return
	 */
	public static Demo getApplication() {
		return application;
	}
}
