/*
 * Copyright 2013 Graham Smith
 */
package uk.co.wobblycogs.samplefx4;

import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.control.ContextMenu;
import javafx.scene.control.TreeCell;
import javafx.scene.control.TreeItem;
import javafx.scene.control.TreeView;
import javafx.scene.control.cell.TextFieldTreeCell;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.ClipboardContent;
import javafx.scene.input.DragEvent;
import javafx.scene.input.Dragboard;
import javafx.scene.input.MouseEvent;
import javafx.scene.input.TransferMode;
import javafx.util.Callback;
import javafx.util.StringConverter;
import org.apache.derby.impl.sql.compile.GetCurrentConnectionNode;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A {@code TreeCell} that supports editing using a text field and context menus
 * on nodes representing departments. Also supports the drag and drop of
 * {@code Person} objects.
 *
 * @param <T>
 * @author Graham Smith
 */
public class EnhancedTextFieldTreeCell<T extends EntityProxy> extends TextFieldTreeCell<EntityProxy> {

	private static Logger logger = LoggerFactory.getLogger(EnhancedTextFieldTreeCell.class);
	public static final Image DEPARTMENT_ICON = new Image(EnhancedTextFieldTreeCell.class.getResourceAsStream("/resources/images/department-24.png"));
	public static final Image PERSON_ICON = new Image(EnhancedTextFieldTreeCell.class.getResourceAsStream("/resources/images/person-24.png"));
	public static final Image TICK_ICON = new Image(EnhancedTextFieldTreeCell.class.getResourceAsStream("/resources/images/tick-24.png"));
	private ContextMenu departmentMenu;

	public EnhancedTextFieldTreeCell(ContextMenu departmentMenu) {
		this.departmentMenu = departmentMenu;
	}

	public EnhancedTextFieldTreeCell(ContextMenu departmentMenu, StringConverter<EntityProxy> sc) {
		super(sc);
		this.departmentMenu = departmentMenu;
	}

	@Override
	public void updateItem(EntityProxy item, boolean empty) {
		super.updateItem(item, empty);

		if (!empty && !isEditing()) {
			setText(getString());
			setGraphic(getTreeItem().getGraphic());
			if (getItem().getType() == EntityProxyType.DEPARTMENT) {
				setContextMenu(departmentMenu);
			}
		}
	}

	private String getString() {
		return getItem() == null ? "" : getItem().toString();
	}

	public static Callback<TreeView<EntityProxy>, TreeCell<EntityProxy>> forDepartmentTreeView(final ContextMenu deptMenu, final DepartmentService departmentService, final PersonService personService) {
		Callback<TreeView<EntityProxy>, TreeCell<EntityProxy>> callback = new Callback<TreeView<EntityProxy>, TreeCell<EntityProxy>>() {
			@Override
			public TreeCell<EntityProxy> call(TreeView<EntityProxy> p) {
				final EnhancedTextFieldTreeCell<EntityProxy> treeCell = new EnhancedTextFieldTreeCell<>(deptMenu);

				//This is required to convert too and from a string for of the
				//tree node when editing. An alternative approach is to just
				//roll your own TreeCell which isn't much more work.
				StringConverter<EntityProxy> converter = new StringConverter<EntityProxy>() {
					@Override
					public String toString(EntityProxy t) {
						return t.getLabel();
					}

					@Override
					public EntityProxy fromString(String label) {
						EntityProxy treeNode = treeCell.getTreeView().getSelectionModel().getSelectedItem().getValue();
						treeNode.setLabel(label);
						return treeNode;
					}
				};
				treeCell.setConverter(converter);

				treeCell.setOnDragDetected(new EventHandler<MouseEvent>() {
					@Override
					public void handle(MouseEvent event) {
						EntityProxy treeNode = treeCell.getItem();
						//Only start the drag and drop operation if the node that is selected
						//is of the correct type.
						if (treeNode.getType() == EntityProxyType.PERSON) {
							Person p = personService.find(treeNode.getId());

							Dragboard dragboard = treeCell.startDragAndDrop(TransferMode.MOVE);
							ClipboardContent content = new ClipboardContent();
							content.put(CompanyManagerController.PERSON_DATA_FORMAT, p.getId());
							dragboard.setContent(content);
							logger.debug("Placed person on the dragboard: {}", p);
						}
						event.consume();
					}
				});

				treeCell.setOnDragOver(new EventHandler< DragEvent>() {
					@Override
					public void handle(DragEvent event) {
						//This tells us which node we are currently hoevering over.
						//We need to know this because only certain nodes at as drop
						//targets for people.
						EntityProxy over = treeCell.getItem();

						//Check that the gesture is of the type expected and that the mouse is
						//over a valid drop target.
						if (event.getDragboard().hasContent(CompanyManagerController.PERSON_DATA_FORMAT)
								&& over != null
								&& over.getType() == EntityProxyType.DEPARTMENT) {
							event.acceptTransferModes(TransferMode.MOVE);
						}
						event.consume();
					}
				});

				treeCell.setOnDragDropped(new EventHandler<DragEvent>() {
					@Override
					public void handle(DragEvent event) {
						EntityProxy over = treeCell.getItem();
						//This is the department that the person will be added to
						Department d = departmentService.find(over.getId());

						Dragboard db = event.getDragboard();
						boolean success = false;
						if (db.hasContent(CompanyManagerController.PERSON_DATA_FORMAT)) {
							//The dragboard contains the ID of a person which is
							//looked up here.
							Person person = personService.find((Long) db.getContent(CompanyManagerController.PERSON_DATA_FORMAT));
							logger.debug("Dropping Person: {}", person);

							//Create the item that will be placed in the tree
							TreeItem<EntityProxy> personItem = new TreeItem<>(new EntityProxy(person.getId(), EntityProxyType.PERSON, person.getName()), new ImageView(PERSON_ICON));
							departmentService.addPerson(d, person);
							treeCell.getTreeItem().getChildren().add(personItem);

							success = true;
						}

						event.setDropCompleted(success);
						event.consume();
					}
				});

				//Used at the end of the drag and drop gesture. If the drag and
				//drop was successful remove the item from it's original location
				//in the tree. The reassignment of the parent of the Person should
				//have been handled by the onDragDropped handler.
				treeCell.setOnDragDone(new EventHandler<DragEvent>() {
					@Override
					public void handle(DragEvent event) {
						if (event.getTransferMode() == TransferMode.MOVE) {
							TreeItem<EntityProxy> item = treeCell.getTreeItem();
							item.getParent().getChildren().remove(item);
							treeCell.getTreeView().getSelectionModel().clearSelection();
						}
						event.consume();
					}
				});

				//Gives the user clear visual feedback that they have found a
				//valid drop target.
				treeCell.setOnDragEntered(new EventHandler<DragEvent>() {
					@Override
					public void handle(DragEvent event) {
						EntityProxy over = treeCell.getItem();
						TreeItem<EntityProxy> item = treeCell.getTreeItem();
						//Only decorate deparment nodes
						if (event.getDragboard().hasContent(CompanyManagerController.PERSON_DATA_FORMAT)
								&& over != null
								&& over.getType() == EntityProxyType.DEPARTMENT) {
							item.setGraphic(new ImageView(TICK_ICON));
							//This is to trigger a refresh of this node in the tree so that
							//the new graphic is drawn.
							item.setValue(new EntityProxy(over.getId(), over.getType(), over.getLabel()));
							//This will also trigger a refresh but it's equally messy.
							//item.setExpanded(false);
							//item.setExpanded(true);
						}
					}
				});

				treeCell.setOnDragExited(new EventHandler<DragEvent>() {
					@Override
					public void handle(DragEvent event) {
						EntityProxy over = treeCell.getItem();
						TreeItem<EntityProxy> item = treeCell.getTreeItem();
						//Only decorate deparment nodes
						if (event.getDragboard().hasContent(CompanyManagerController.PERSON_DATA_FORMAT)
								&& over != null
								&& over.getType() == EntityProxyType.DEPARTMENT) {
							item.setGraphic(new ImageView(DEPARTMENT_ICON));
							//This is needed to catch the situation where the user drags to
							//a location that isn't over a valid drag target such as off
							//the bottom of the tree.
							item.setValue(new EntityProxy(over.getId(), over.getType(), over.getLabel()));
						}
					}
				});

				return treeCell;
			}
		};

		return callback;
	}
}
