package uk.co.wobblycogs.samplefx4;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

/**
 * Used to aquire an {@code EmtityManagerFactory} given a particular persistence
 * unit name. Only one EntityManagerFactory is required for each persistence
 * unit and it is wasteful to keep recreating it. Each EntityManagerFactory can
 * produce multiple {@code EntityManager} instances.
 *
 * @author Graham Smith
 */
public final class PersistenceUtilities {

	private static volatile PersistenceUtilities instance;
	private Map<String, EntityManagerFactory> emfMap = new HashMap<>();

	private PersistenceUtilities() {
	}

	public static PersistenceUtilities getInstance() {
		//double-checked lock - safe from Java 5 onwards.
		if (instance == null) {
			synchronized (PersistenceUtilities.class) {
				if (instance == null) {
					instance = new PersistenceUtilities();
				}
			}
		}
		return instance;
	}

	public boolean isConfigured(String persistenceUnitName) {
		return emfMap.containsKey(persistenceUnitName);
	}

	/**
	 * TODO: This currently only supports loading settings for a single
	 * persistence unit. It should prefix the settings name with the name of the
	 * persistence unit to support multiple configurations.
	 *
	 * @param persistenceUnitName the name of the persistence unit to create.
	 * @param emfSettings a map of settings passed directly to the factory
	 * creator.
	 * @return
	 * @throws IOException
	 */
	public EntityManagerFactory getEntityManagerFactory(String persistenceUnitName, Map<String, String> emfSettings) {
		if (emfMap.get(persistenceUnitName) == null) {
			synchronized (this) {
				if (emfMap.get(persistenceUnitName) == null) {
					EntityManagerFactory emf = null;
					if (emfSettings == null) {
						emf = Persistence.createEntityManagerFactory(persistenceUnitName);
					} else {
						emf = Persistence.createEntityManagerFactory(persistenceUnitName, emfSettings);
					}
					emfMap.put(persistenceUnitName, emf);
				}
			}
		}
		return emfMap.get(persistenceUnitName);
	}
}
